rm(list=ls())

###########################
#REPRESENTACIN DE DATOS
###########################

#cargar los datos.
datos_temp <- read.table("datos.txt", header=TRUE, quote="\"", comment.char="%")
datos_CO2<- read.csv("climate-change.csv")

#se seleccionan los datos que interesan
datos_conjunto <- merge(x=datos_temp[datos_temp$Year>=1954,],y=subset(x = datos_CO2, subset = Entity =="World", select = c("Year", "CO2.concentrations")), by=c("Year")) #seleccionamos a partir de 1954 que es a partir del ao que se tienen datos de CO2 para todos los aos


library(ggplot2)
library(ggridges)
library(viridisLite)
library(viridis)

#Representacion de las funciones de densidad de los suavizados de las distribuciones
ggplot(datos_conjunto, aes(x = `Anomaly`, y = as.factor(Year), fill = `CO2.concentrations`)) +
  geom_density_ridges_gradient(scale = 30, rel_min_height = 0.01) +
  scale_x_continuous("Anomala / C",expand=c(0,0)) +
  scale_y_discrete("Ao",breaks=c(1954,1955,1960,1965,1970,1975,1980,1985,1990,1995,2000,2005,2010,2015,2020,2021)) +
  scale_fill_viridis_c(name = "CO2 / ppm", option = "C",direction=-1) +
  coord_cartesian(clip = "off") +
  theme_ridges(font_size = 18, grid = TRUE) 


#################################
#OBTENCION DEL PARAMETRO h
#################################

#Se inicializan las variables
anos<-datos_conjunto$Year[1]:datos_conjunto$Year[length(datos_conjunto$Year)] #anos 
valores_CO2<-subset(x = datos_CO2, Entity =="World" & Year >= anos[1] & Year <= anos[length(anos)], select = c("Year", "CO2.concentrations")) #valores de CO2 de los aos que analizamos
d_ij<-matrix(data=NA,nrow=length(anos),ncol=length(anos)) #matriz con todas las posibles diferencias dos a dos entre los valores de CO2 de todos los aos
minj_d_ij<- vector(length=length(anos)) #en cada fila de este vector se guarda el minimo de las diferencias de CO2 para un ao i

#Representacion valores CO2
plot(valores_CO2$Year,valores_CO2$CO2.concentrations,xlab="Ao",ylab="CO2 / ppm",cex.lab=1.5)


#Se calculan las diferencias de CO2 entre los aos.
for(i in 1:length(anos)){
  for(j in (1:length(anos))[-i]){
    d_ij[i,j]=abs(valores_CO2$CO2.concentrations[valores_CO2$Year==(i+anos[1]-1)]-valores_CO2$CO2.concentrations[valores_CO2$Year==j+anos[1]-1])
  }
  minj_d_ij[i]<-min(d_ij[i,],na.rm=TRUE)
}
h_m<-max(minj_d_ij) #primera h que se probara para encontrar h optima
h_M<- (min(d_ij,na.rm = TRUE)+max(d_ij,na.rm=TRUE))/2 #ultima h que se probara
hs<-seq(h_m,h_M,(h_M-h_m)/10) #valores de h que se probaran desde hm hasta hM, equidistantes.

#Inicializacion de variables para el calculo de h optima
probs<-c(1:250)/251 #rdenes cuantiles que se calculan
suma_num<-vector(length=length(probs)) #numerador de la estimacion de los cuantiles
suma_den<-vector(length=length(probs)) #denominador de la estimacion de los cuantiles
cuantiles_estimados<-vector(length=length(probs)) #vector donde se guardan las estimaciones de los cuantiles
distancia_Wasserstein<-0 
anos_hs_dw <- data.frame(matrix(c(rep(anos,each=length(hs)),rep(hs,length(anos)),rep(NA,length(anos)*length(hs))),nrow=length(anos)*length(hs),ncol=3)) #matriz donde se guardan los errores de cada ao y cada valor de h
names(anos_hs_dw) <- c("Ano","h","Distancia_Wasserstein")

valores_cuantil<-data.frame(matrix(data=NA,nrow=length(anos)*length(probs),ncol=4)) #una columna para el ao, otra para las probs, otra para los cuantiles y otra para las estimaciones de los cuantiles. 
valores_cuantil$Ano<-rep(anos,each=length(probs)) #se rellena la columna de los aos
valores_cuantil$Probabilidad<-rep(probs,length(anos)) #se rellena la columna de las probs
#se calculan los cuantiles
for(j in anos){
  valores_cuantil$`Cuantil dato`[valores_cuantil$Ano==j]<-quantile(datos_conjunto$Anomaly[datos_conjunto$Year==j],probs)
}

#funcion para la obtencion del parametro del error para cada ao y cada valor de h
cv=function(t){
  ###Obtencin del parmetro h
  ano<-anos_hs_dw[t,1]
  h<-anos_hs_dw[t,2]

  for(k in 1:length(probs)){ #se calcula cada cuantil
    for(j in anos[-c(ano-(anos[1]-1))]){ #aos para calcular cada cuantil. se quita el ano "ano"
      #cuantiles_ano_j <- quantile(datos_conjunto[datos_conjunto$Year==j,]$Anomaly,probs)
      suma_num[k] <- suma_num[k]+1/(sqrt(2*pi))*exp(-(((d_ij[ano-(anos[1]-1),j-(anos[1]-1)])/h)^2)/2)*valores_cuantil$`Cuantil dato`[valores_cuantil$Ano==j][k]
      suma_den[k] <- suma_den[k]+1/(sqrt(2*pi))*exp(-(((d_ij[ano-(anos[1]-1),j-(anos[1]-1)])/h)^2)/2)
    }
    cuantiles_estimados[k]<-suma_num[k]/suma_den[k]
    suma_num<-rep(0,length(probs))
    suma_den<-rep(0,length(probs))
  }

  distancia_Wasserstein <- sum((valores_cuantil$`Cuantil dato`[valores_cuantil$Ano==ano]-cuantiles_estimados)^2*probs[1]) #multiplicamos por probs[1] porque las probs estn equiespaciadas
  return(distancia_Wasserstein)
}

anos_hs_dw$Distancia_Wasserstein<-unlist(lapply(1:(length(anos)*length(hs)),cv))

#se calcula el error de cada h
errores <- data.frame(matrix(c(hs,rep(NA,length(hs))),nrow=length(hs),ncol=2))
names(errores)<-c("h","Error")

for(i in 1:length(hs)){
  errores$Error[i]<-sum((anos_hs_dw$Distancia_Wasserstein[anos_hs_dw$h==hs[i]])^2)
}
h_optima<- anos_hs_dw$h[which.min(errores$Error)] #h optima
plot(errores$h,errores$Error,xlab="h",ylab="Error") #grafico de los errores

#se ponen 0s en la diagonal de la matriz de distancias entre valores de CO2 para que no de problemas en la siguiente funcion
for(i in 1:length(anos)){
  d_ij[i,i]=0
}

#####################################
#ESTIMACION DE LOS CUANTILES
####################################

#funcion para la estimacion de los cuantiles
prediccion_cuantiles=function(ano){
  for(k in 1:length(probs)){
    for(j in anos){
      suma_num[k] <- suma_num[k]+1/(sqrt(2*pi))*exp(-(((d_ij[ano-(anos[1]-1),j-(anos[1]-1)])/h_optima)^2)/2)*valores_cuantil$`Cuantil dato`[valores_cuantil$Ano==j][k]
      suma_den[k] <- suma_den[k]+1/(sqrt(2*pi))*exp(-(((d_ij[ano-(anos[1]-1),j-(anos[1]-1)])/h_optima)^2)/2)
    }
    cuantiles_estimados[k]<-suma_num[k]/suma_den[k]
    suma_num<-rep(0,length(probs))
    suma_den<-rep(0,length(probs))
  }
  cuantiles_estimados
}

valores_cuantil$`Cuantil estimado`<-unlist(lapply(anos,prediccion_cuantiles))

#se representan las funciones cuantil

ggplot(valores_cuantil,aes(x = Probabilidad, y = `Cuantil estimado`,group=Ano)) +
  geom_line(aes(color=Ano))+
  scale_x_continuous("Probabilidad")+
  scale_y_continuous("Cuantil / C")+
  scale_color_viridis(option="D",name="Ao")+  
  theme_ridges(font_size = 18, grid = TRUE)



  
#Calculo percentiles 5, 10, 25, 50, 75 y 90.
#inicializacion de variables
probs<- c(0.05,0.1,0.25,0.5,0.75,0.9)
valores_cuantil<-data.frame(matrix(data=NA,nrow=length(anos)*length(probs),ncol=4)) #una columna para el ao, otra para las probs y otra para los cuantiles. En las 232 primeras filas el primer cuantil, en las 232 siguientes, el segundo cuantil... Para poder luego hacer un grfico ggplot
names(valores_cuantil)<-c("Ano","Probabilidad","Cuantil dato", "Cuantil estimado") #nombres de las columnas del df
valores_cuantil$Ano<-rep(anos,each=length(probs)) #se rellena la columna de los aos
valores_cuantil$Probabilidad<-rep(probs,length(anos)) #se rellena la columna de las probs
for(j in anos){
  valores_cuantil$`Cuantil dato`[valores_cuantil$Ano==j]<-quantile(datos_conjunto$Anomaly[datos_conjunto$Year==j],probs)
}
suma_num<-vector(length=length(probs)) #numerador de la estimacion de los cuantiles
suma_den<-vector(length=length(probs)) #denominador de la estimacion de los cuantiles
cuantiles_estimados<-vector(length=length(probs))
#funcion para obtener las estimaciones de los cuantiles
valores_cuantil$`Cuantil estimado`<-unlist(lapply(anos,prediccion_cuantiles))

#grafico de los cuantiles
plot(valores_cuantil$`Cuantil estimado`, valores_cuantil$Ano, xlab="Percentil / C", ylab="Ao",type='n')
for(i in 1:length(probs)){
  points(valores_cuantil$`Cuantil estimado`[valores_cuantil$Probabilidad==probs[i]],anos,col=i,pch=16,cex=0.5)
}
legend('bottomright', title='Porcentajes',legend=probs*100, col = c(1:length(probs)),pch=16,cex=1,text.width = 0.1,pt.cex=1,y.intersp=0.6,pt.lwd=2,x.intersp=0.3,ncol=2,bty='n')

